from datetime import datetime
from typing import List, Optional, Dict, Any

from janis_core import *
from janis_core.types.common_data_types import Boolean, File, String, Int

Rsat_Dyad_Analysis_V0_1_0 = CommandToolBuilder(tool="rsat_dyad_analysis", base_command=["rsat", "dyad-analysis"], inputs=[ToolInput(tag="in_help", input_type=Boolean(optional=True), prefix="-help", doc=InputDocumentation(doc="(must be first argument) display options")), ToolInput(tag="in_sequence_analyzed_sequences", input_type=File(optional=True), prefix="-o", doc=InputDocumentation(doc="The sequence to be analyzed. Multiple sequences can\nbe entered at once with most sequence formats (see below).")), ToolInput(tag="in_mask", input_type=String(optional=True), prefix="-mask", doc=InputDocumentation(doc="|lower\nMask lower or uppercases, respecively, i.e. replace\nselected case by N characters.")), ToolInput(tag="in_format", input_type=String(optional=True), prefix="-format", doc=InputDocumentation(doc="Input sequence format. Various standards are\nsupported.\nraw: the raw sequence without any identifier or comment.\nmulti: several raw sequences concatenated.\nIG: IntelliGenetics format.\nFastA: the sequence format used by FastA, BLAST, Gibbs\nsampler and a lot of other bioinformatic programs.\nWconsensus: the format defined by Jerry Hertz for\nhis programs (patser, consensus, wconsensus).")), ToolInput(tag="in_oligolengtholigonucleotide_size_default", input_type=Boolean(optional=True), prefix="-l", doc=InputDocumentation(doc="#    oligo_length\nOligonucleotide size (default 3)\nThis is the size of a single element (a half dyad).")), ToolInput(tag="in_sp", input_type=Boolean(optional=True), prefix="-sp", doc=InputDocumentation(doc="#-# spacing (default 0-20)\nSpacing between the elements of the dyad.\nThe spacing is the number of bases between the end of\nthe first element and the start of the second one.\nSpacing formats\n---------------\nA single integer value means that the spacing is fixed.\nVariable spacing can be introdued by entering the min and\nmax values separated by a hyphen.\nFor example 8-12 means that all occurrences of the dyad\nwith a spacing between 8 and 12 qill be counted together\nand their significance estimated globally.\nWarning, this is different from scanning one by one the\nspacing values 8 to 12.")), ToolInput(tag="in_type", input_type=String(optional=True), prefix="-type", doc=InputDocumentation(doc="(dr|ir|any)\nIn order to fasten execution, the program can be asked\nto restrict its analysis to symmetric dyads.\nThree types are accepted\ndr   direct repeats: the second element is the same as the\nfirst one\nir   inverted repeats: the second element is the reverse\ncomplement of the first one.\nrep  repeats: direct and inverted repeats are evaluated\nany  (default)\nWhen selecting the option any, the analysis is\nperformed on all non-symmetric dyads as well.")), ToolInput(tag="in_accept", input_type=File(optional=True), prefix="-accept", doc=InputDocumentation(doc="Specify a file containing a list of accepted\ndyads. The analysis is then restricted to these\ndyads. Since the number of tested dyads is reduced by\nthis selection, the multitesting correction is\ngenerally lower, which increases the significance of\nthe accepted dyads, compared to the default situation\nwhere all dyads are analyzed.\nFile format: the first word of each row specifies a\ndyad. Subsequent words are ignored.")), ToolInput(tag="in_groups_p", input_type=Boolean(optional=True), prefix="-groupsp", doc=InputDocumentation(doc="Group dyads made of the same words (monads) but with\ndifferent spacings.")), ToolInput(tag="in_two_str", input_type=Boolean(optional=True), prefix="-2str", doc=InputDocumentation(doc="count on oth strands\nThe occurrences of each oligonucleotide are summed on both\nstrands. This allows to detect elements which act in an\norientation-insensitive way (as is generally the case for\nyeast upstream elements).")), ToolInput(tag="in_one_str", input_type=Boolean(optional=True), prefix="-1str", doc=InputDocumentation(doc="single strand count\nonly the direct strand is considered for oligonucleotide and\ndyad occurrence counting.")), ToolInput(tag="in_prot", input_type=Boolean(optional=True), prefix="-prot", doc=InputDocumentation(doc="input sequence is proteic. In this case, the analysis\nconcerns pairs of oligopeptides instead of oligonucleotides")), ToolInput(tag="in_exp_freq", input_type=Boolean(optional=True), prefix="-expfreq", doc=InputDocumentation(doc="file with an expected frequency table\nBy default, the frequency expected for each dyad is the\nproduct of the frequency expected for each element\n(oligonucleotide):\nexp(dyad) = exp(oligo1)*exp(oligo2)\nBy default, the oligonucleotide frequencies observed in the\ninput sequences are used to estimate the expected oligo\nfrequencies.\nAlternatively, predefined frequency tables can be used.\nThese tables can for instance be calculated on basis of\n- the whole yeast genome\n- all yeast intergenic regions\n- all yeast gene regions\nThis allows to correct the bias due to the highly variable\ndistribution of oligonucleotides observed in the yeast genome.")), ToolInput(tag="in_ncf", input_type=Boolean(optional=True), prefix="-ncf", doc=InputDocumentation(doc="(deprecated, use '-bg intergenic' instead)\nuse intergenic frequencies as background frequencies")), ToolInput(tag="in_bg", input_type=Boolean(optional=True), prefix="-bg", doc=InputDocumentation(doc="background model\nType of sequences used as background model for\nestimating expected dyad frequencies.\nEither -org or -taxon is required with the option -bg.\nSupported models:\n-bg upstream\nall upstream sequences, allowing overlap with\nupstream ORFs\n-bg upstream-noorf\nall upstream sequences, preventing overlap\nwith upstream ORFs\n-bg intergenic\nintergenic frequencies\nWhole set of intergenic regions, including\nupstream and downstream sequences\n-bg monads (default)\nCalcualte expected dyad frequency from the monad\nfrequencies observed in the input sequences.\n-bg input\nSame as -bg monads, allowed for consistency with\noligo-analysis.")), ToolInput(tag="in_org", input_type=Boolean(optional=True), prefix="-org", doc=InputDocumentation(doc="organism")), ToolInput(tag="in_tax_on", input_type=Boolean(optional=True), prefix="-taxon", doc=InputDocumentation(doc="taxon\nOrganism or taxon that used as reference for the\nestimation of a background model based on a genome\nsubset (option -bg).  Either -org or -taxon is\nrequired with the option -bg.\nOptions -org and -taxon are mutually exclusive.")), ToolInput(tag="in_tho_sig", input_type=Boolean(optional=True), prefix="-thosig", doc=InputDocumentation(doc="#\nThreshold on occurrence significance.\n(obsolete: use -lth occ_sig instead)")), ToolInput(tag="in_lth", input_type=Int(optional=True), prefix="-lth", doc=InputDocumentation(doc="value\nLower threshold on some parameter. All patterns with a\nparameter value smaller than the threshold are\ndiscarded.\nSupported parameters: occ,occ_P,occ_E,occ_sig,observed_freq,exp_freq,zscore,mseq,ms_P,ms_E,ms_sig,ratio,rank\nExample: select patterns with a positive value for the\noccurrence significance.\n-lth occ_sig 0")), ToolInput(tag="in_uth", input_type=Int(optional=True), prefix="-uth", doc=InputDocumentation(doc="value\nUpper threshold on some parameter. All patterns with a\nparameter value higher than the threshold are\ndiscarded.\nSupported parameters: occ,occ_P,occ_E,occ_sig,observed_freq,exp_freq,zscore,mseq,ms_P,ms_E,ms_sig,ratio,rank\nExample: to select no more than 50 patterns\n-uth rank 50")), ToolInput(tag="in_sort", input_type=Boolean(optional=True), prefix="-sort", doc=InputDocumentation(doc="sort results by decreasing order of significance")), ToolInput(tag="in_return", input_type=String(optional=True), prefix="-return", doc=InputDocumentation(doc="output fields may contain one or several of the following\nwords:\nfreq\nocc\nproba (binomial probability)\nzscore\nratio\nrank\nthe fields have to be separated by commas\nBy default, only occurrences are returned.")), ToolInput(tag="in_under", input_type=Boolean(optional=True), prefix="-under", doc=InputDocumentation(doc="detect under-represented instead of over-represented\ndyads (left-tail significance test).")), ToolInput(tag="in_two_tails", input_type=Boolean(optional=True), prefix="-two_tails", doc=InputDocumentation(doc="detect under-represented and over-represented dyads\n(two-tail significance test).")), ToolInput(tag="in_zero_occ", input_type=Boolean(optional=True), prefix="-zeroocc", doc=InputDocumentation(doc="Report also dyads with zero occurrences (provided they\nfit the other thresholds).  By default, the program\nreports only patterns present in the sequence.  If the\nleft tail or two-tail test is applied, patterns with\nzero occurrences are automatically taken into\naccount.\nIn some other cases, one would also like to detect\npatterns absent from the sequence. This is the\nfunction of the option -zeroocc.")), ToolInput(tag="in_quick", input_type=Boolean(optional=True), prefix="-quick", doc=InputDocumentation(doc="Quick count mode: delegate the counting of word\noccurrences to count-words, a program written in C by\nMatthieu Defrance.\nThis option is incompatible with the following output\nfields:")), ToolInput(tag="in_noov", input_type=Boolean(optional=True), prefix="-noov", doc=InputDocumentation(doc="do not allow overlapping matches of the same word")), ToolInput(tag="in_timeout", input_type=Boolean(optional=True), prefix="-timeout", doc=InputDocumentation(doc="#\ntimeout (in seconds). Default = 3600.\ndyad-analysis can be time consuming. In order to\nprotect the server from endless queries, the program\nwill automatically stop after 1 hour (default) of\ncalculation. The time out value can be changed for\nheavy tasks.")), ToolInput(tag="in_seqtype", input_type=Boolean(optional=True), prefix="-seqtype", doc=InputDocumentation(doc="dna|prot|other\nInput sequence type\n. DNA (default)\nOnly A, C, G, and T residues are\naccepted. oligomers that contain partly defined\n(IUPAC code) or undefined nucleotides (N) are\ndiscarded from the countings.\n. protein\nOligopeptide analysis instead of oligonucleotide.\nThis inactivates the grouping of oligomers with\ntheir reverse complements, and modifies the\nalphabet size.\n. other\nAny type of letters found in the input sequence is\nconsidered valid. This allows to analyze texts in\nhuman language.")), ToolInput(tag="in_statistics", input_type=String(), position=0, doc=InputDocumentation(doc="sequences")), ToolInput(tag="in_pattern_discovery", input_type=String(), position=1, doc=InputDocumentation(doc="OPTIONS")), ToolInput(tag="in_oligonucleotide_dot", input_type=String(), position=0, doc=InputDocumentation(doc="Sig value higher than 0")), ToolInput(tag="in_pxx_sum_pj", input_type=Int(), position=0, doc=InputDocumentation(doc="P(X>=x) =  1 - SUM P(j)")), ToolInput(tag="in_accuracy_dot", input_type=String(), position=0, doc=InputDocumentation(doc="Interpretation: the P-value (Pval) represents the nominal risk")), ToolInput(tag="in_pxx_sum_pxi", input_type=Int(), position=0, doc=InputDocumentation(doc="P(X<=x) =  SUM P(X=i)")), ToolInput(tag="in_e_value", input_type=String(), position=0, doc=InputDocumentation(doc="The probability of occurrence by itself is not fully\ninformative, because the threshold must be adapted depending\non the number of patterns considered. Indeed, a simple\nhexanucleotide analysis amounts to consider 4096\nhypotheses.\nThe E-value (Eval) represents the expected number of patterns\nwhich would be returned at random for a given threshold of\nnominal P-value.\nEval = NPO * P(X>=x)\nwhere NPO is the number of possible oligomers of the chosen\nlength (eg 4096 for hexanucleotides).\nNote that when searches are performed on both strands, NPO is\ncorrected for the fact that non-palindromic patterns are\ngrouped by pairs (for example, there are 4096 possible\nhexanucleotides when the count is performed on a single\nstrand, but only 2080 when the count is performed on both\nstrands).\nInterpretation: the E-value (Eval) represents the expected\nnumber of false positives, for a given threshold of P-value\n(Pval)."))], outputs=[ToolOutput(tag="out_sequence_analyzed_sequences", output_type=File(optional=True), selector=InputSelector(input_to_select="in_sequence_analyzed_sequences", type_hint=File()), doc=OutputDocumentation(doc="The sequence to be analyzed. Multiple sequences can\nbe entered at once with most sequence formats (see below)."))], container=None, version="v0.1.0")


if __name__ == "__main__":
    # or "cwl"
    Rsat_Dyad_Analysis_V0_1_0().translate("wdl", allow_empty_container=True)

